import json
import shutil
from pathlib import Path
from . import TEMPLATE_DIR
from nyvalis.tools import log
from typing import Dict, Any, List
from ...models import DependencyObj, FeaturePath

logger = log.get(log.LIB_NAME)

BASE_TYPESCRIPT_DIR = Path(TEMPLATE_DIR) / "typecheck" / "typescript"
BASE_JAVASCRIPT_DIR = Path(TEMPLATE_DIR) / "typecheck" / "javascript"

class TypecheckEnum:
    TYPESCRIPT = DependencyObj(title="Typescript", value="ts", 
                               packages={"typescript": "~5.7.2", "typescript-eslint": "^8.22.0",}, extras={})
    

    JSDOCS = DependencyObj(title="JSDoc comments", value="js",
                            packages={}, extras={})
    

    JAVASCRIPT = DependencyObj(title="Vanilla Javascript", value="js",
                                packages={}, extras={})


def resolve(path: str, typecheck: DependencyObj, features: List[DependencyObj]):
    use_typecheck = typecheck.value != "js"

    extras = {}
    dependencies = {}
    filepaths: Dict[str, FeaturePath] = {}

    if use_typecheck:
        dependencies.update(TypecheckEnum.TYPESCRIPT.packages)
        filepaths.update({
            "App.tsx": FeaturePath(BASE_TYPESCRIPT_DIR / "App.tsx", Path(path) / "src" / "App.tsx"),
            "main.tsx": FeaturePath(BASE_TYPESCRIPT_DIR / "main.tsx", Path(path) / "src" / "main.tsx"),
            "tsconfig.json": FeaturePath(BASE_TYPESCRIPT_DIR / "tsconfig.json", Path(path) / "tsconfig.json"),
            "tsconfig.app.json": FeaturePath(BASE_TYPESCRIPT_DIR / "tsconfig.app.json", Path(path) / "tsconfig.app.json"),
            "tsconfig.node.json": FeaturePath(BASE_TYPESCRIPT_DIR / "tsconfig.node.json", Path(path) / "tsconfig.node.json"),
            "vite-env.d.ts": FeaturePath(BASE_TYPESCRIPT_DIR / "vite-env.d.ts", Path(path) / "src" / "vite-env.d.ts"),
            "vite.config.ts": FeaturePath(BASE_TYPESCRIPT_DIR / "vite.config.ts", Path(path) / "vite.config.ts"),
            "index.html": FeaturePath(BASE_TYPESCRIPT_DIR / "index.html", Path(path) / "index.html"),
        })
    else:
        dependencies.update(TypecheckEnum.JAVASCRIPT.packages)
        filepaths.update({
            "App.jsx": FeaturePath(BASE_JAVASCRIPT_DIR / "App.jsx", Path(path) / "src" / "App.jsx"),
            "main.jsx": FeaturePath(BASE_JAVASCRIPT_DIR / "main.jsx", Path(path) / "src" / "main.jsx"),
            "vite.config.js": FeaturePath(BASE_JAVASCRIPT_DIR / "vite.config.js", Path(path) / "vite.config.js"),
            "index.html": FeaturePath(BASE_JAVASCRIPT_DIR / "index.html", Path(path) / "index.html"),
        })
        
    logger.info("Copying typecheck files")
    for filepath in filepaths.values():
        filepath.dst.parent.mkdir(parents=True, exist_ok=True)
        shutil.copy(filepath.src, filepath.dst)

    logger.info("Writing package.json")
    json_path = Path(path) / "package.json"
    with open(json_path, 'r') as file:
        config: Dict[str, Any] = json.load(file)

    config["name"] = path if path != "." else "My app"
    config["devDependencies"].update(dependencies)

    with open(json_path, 'w') as file:
        json.dump(config, file, indent=2)

    logger.info("package.json ready")



import os
import psutil
import requests
import subprocess
from time import sleep
from typing import Callable, List
from nyvalis.tools import stdout, log
from nyvalis.core.config import Config
from nyvalis.core.invoker import Invoker
from nyvalis.core import winforms, CONF_FILENAME

from System.Windows.Forms import Application
from System.Threading import ApartmentState, Thread, ThreadStart

class NyvalisClientError(Exception):
    """Exception for client"""
    pass


class Builder:
    config: Config
    logger = log.get(log.LIB_NAME)
    _workers = 2
    _state = None

    @classmethod
    def default(cls):
        from nyvalis.core.config.exceptions import NyvalisConfigError
        try:
            stdout.info("Starting application")
            cls.logger.info(f"Starting builder")
            
            conf = os.path.join(os.getcwd(), CONF_FILENAME)
            
            if "__compiled__" in globals():
                from nyvalis.core import ROOT_PATH
                conf = os.path.join(ROOT_PATH, CONF_FILENAME)

            cls.logger.info(f"Loading {CONF_FILENAME}")
            Builder.config = Config.read(conf, "utf-8")
            return Builder()
        except NyvalisConfigError as err:
            stdout.error(f"{err}", exit=True)

    @classmethod
    def state(cls, object):
        cls._state = object
        return Builder
    
    @classmethod
    def run(cls) -> None:
        client_process = None
        
        try:
            if not "__compiled__" in globals():
                client_process = Builder.wait_client()

                if not client_process:
                    stdout.error("Failed to start client")
                    raise NyvalisClientError("Failed to start client")

            Application.EnableVisualStyles()
            Application.SetCompatibleTextRenderingDefault(False)

            thread = Thread(ThreadStart(lambda: winforms.initialize(cls.config, cls._workers, cls._state)))
            thread.SetApartmentState(ApartmentState.STA)
            thread.Start()
            thread.Join()
        except Exception as err:
            stdout.error(f"Failed running builder: {err}")
            cls.logger.critical(f"Failed running builder: {err}")
        finally:
            if client_process is not None:
                Builder.kill_processes(client_process.pid)

    @classmethod
    def invokers(cls, invokers: List[Callable]):
        for invoker in invokers:
            name = invoker.__name__
            if Invoker.commands.get(name):
                Invoker.commands[name].allowed = True
                cls.logger.info(f"Registering command: `{name}` ")
                
        return Builder()
    
    @classmethod
    def workers(self, amount: int) -> "Builder":
        self.workers = amount
        return Builder

    
    @classmethod
    def kill_processes(cls, parent_pid):
        parent = psutil.Process(parent_pid)
        for child in parent.children(recursive=True):
            child.kill()
        cls.logger.info(f"Client terminated")
        log.Env.__del__(log.Env)
        cls.logger.info(f"QueueListener stopped")


    @classmethod
    def wait_client(cls) -> None:
        client_online = False
        while True:
            try:
                cls.logger.info(f"Checking {Builder.config.build.dev_url} status")
                response = requests.head(Builder.config.build.dev_url)
                if response.status_code == 200:
                    if not client_online:
                        client_online = True
                    return
            except requests.RequestException:
                if not client_online:
                    cls.logger.info("Starting frontend process")
                    process = subprocess.Popen(Builder.config.build.before_dev_command, shell=True, stdout=subprocess.DEVNULL, 
                    stderr=subprocess.DEVNULL)
                    cls.logger.info("Success")
                    client_online = True
                    cls.logger.info("Returning process")
                    return process
            sleep(0.25)


import sys
import click
from nyvalis.core import LIB_NAME

def info(msg: str, exit: bool = False):
    click.echo(f"\033[36;2m[{LIB_NAME}] \033[0m{msg}")

    if exit:
        sys.exit(0)

def error(msg: str, exit: bool = False):
    click.echo(f"\033[36;2m[{LIB_NAME}] \033[31m{msg}\033[0m")

    if exit:
        sys.exit(1)


import click
from nyvalis.cli.commands import run, new, build

@click.group()
def main():
    pass

main.add_command(new, name="new")
main.add_command(run, name="run")
main.add_command(build, name="build")


if __name__ == "__main__":
    main()


import json
from dataclasses import dataclass
from nyvalis.core.config.exceptions import NyvalisConfigError

@dataclass
class WindowConfig:
    title: str
    width: float
    height: float
    resizable: bool
    min_height: float
    min_width: float


@dataclass
class BuildConfig:
    before_dev_command: str
    before_build_command: str
    dev_url: str
    dist_dir: str


@dataclass
class Config:
    project_name: str
    version: str
    identifier: str
    icon: str
    context_menu: bool
    dev_tools: bool
    build: BuildConfig
    window: WindowConfig

    def __init__(self, config: dict):
        self.validate_config(config)

        self.project_name = config['productName']
        self.version = config['version']
        self.identifier = config['identifier']
        self.icon = config['icon']
        self.dev_tools = config['devTools']
        self.context_menu = config['contextMenu']
        
        self.build = BuildConfig(
            before_dev_command=config['build']['beforeDevCommand'],
            before_build_command=config['build']['beforeBuildCommand'],
            dev_url=config['build']['devUrl'],
            dist_dir=config['build']['distDir']
        )

        self.window = WindowConfig(
            title=config['window']['title'],
            width=config['window']['width'],
            height=config['window']['height'],
            resizable=config['window']['resizable'],
            min_height=config['window']['minHeight'],
            min_width=config['window']['minWidth']
        )


    @staticmethod
    def validate_config(config: dict):
        required_fields = {
            "productName": str,
            "version": str,
            "identifier": str,
            "icon": str,
            "devTools": bool,
            "contextMenu": bool,
            "build": dict,
            "window": dict,
        }

        build_fields = {
            "beforeDevCommand": str,
            "beforeBuildCommand": str,
            "devUrl": str,
            "distDir": str,
        }

        window_fields = {
            "title": str,
            "width": (int, float),
            "height": (int, float),
            "resizable": bool,
            "minHeight": (int, float),
            "minWidth": (int, float),
        }

        for key, expected_type in required_fields.items():
            if key not in config:
                raise NyvalisConfigError(f"Missing required field: {key}")
            if not isinstance(config[key], expected_type):
                raise NyvalisConfigError(f"Incorrect type for {key}: expected {expected_type}, got {type(config[key])}")
            if isinstance(config[key], str) and not config[key]:
                if key == "icon":
                    continue
                raise NyvalisConfigError(f"Field '{key}' cannot be empty")

        for key, expected_type in build_fields.items():
            if key not in config["build"]:
                raise NyvalisConfigError(f"Missing required field in 'build': {key}")
            if not isinstance(config["build"][key], expected_type):
                raise NyvalisConfigError(f"Incorrect type for build.{key}: expected {expected_type}, got {type(config['build'][key])}")

        for key, expected_type in window_fields.items():
            if key not in config["window"]:
                raise NyvalisConfigError(f"Missing required field in 'window': {key}")
            if not isinstance(config["window"][key], expected_type):
                raise NyvalisConfigError(f"Incorrect type for window.{key}: expected {expected_type}, got {type(config['window'][key])}")

        if config["window"]["width"] <= 0:
            raise NyvalisConfigError("Width must be greater than 0")
        if config["window"]["height"] <= 0:
            raise NyvalisConfigError("Height must be greater than 0")
        
    @staticmethod
    def read(path: str, encoding: str = "utf-8"):

        try:
            with open(path, encoding=encoding) as config_file:
                config_data = json.load(config_file)
                return Config(config_data)
        except Exception as err:
            raise NyvalisConfigError(f"Failed to load config file: {err}")



import os
import sys
import json
import time
import click
import subprocess
import questionary
from pathlib import Path
from nyvalis import bundler
from nyvalis.tools import log
from nyvalis.tools import stdout
from nyvalis.cli.commands import utils
from nyvalis.core import CONF_FILENAME, LIB_NAME

ENTRY_FOLDER_NAME = "src-python"
BUILD_FOLDER_NAME = "releases"
ENTRY_POINT_PATH = os.path.join(ENTRY_FOLDER_NAME, "main.py")
CONFIG_PATH = os.path.join(ENTRY_FOLDER_NAME, CONF_FILENAME)

logger = log.get(log.LIB_NAME)

@click.command()
@click.option('--debug', is_flag=True, help="Enable console terminal")
@click.option('--verbose', '-v', is_flag=True, help="Change lib logger level to DEBUG")
@click.option('--skip-installer', is_flag=True, help="Skip the building of installer")
def build(debug: str, verbose, skip_installer):
    """Builds a existing project.
    """

    if verbose:
        logger.setLevel(log.DEBUG)

    if not os.path.isdir("src-python"):
        logger.info("Could not find `src-python`, changing directory")
        os.chdir(Path(os.path.dirname(os.getcwd())).resolve())
        if not os.path.isdir("src-python"):
            logger.error("Could not find application.")
            stdout.error("Could not find application.", exit=True)

    if not os.path.isfile(ENTRY_POINT_PATH):
        logger.error("Could not find `main.py`")
        stdout.error("Could not find application.", exit=True)

    if not os.path.isfile(CONFIG_PATH):
        logger.error(f"Could not find `{CONF_FILENAME}`")
        stdout.error("Could not find application.", exit=True)

    is_installable = None
    if not skip_installer:
        is_installable = utils.prompt_select(title=f"Create installer?", choices=[questionary.Choice(title="Yes", value=True),
            questionary.Choice(title="No", value=False)])
    
    logger.info("Reading config file")
    with open(CONFIG_PATH, "r") as file:
        config = json.load(file)


    version = config["version"]
    product_name = config["productName"]
    start = time.perf_counter()


    build_frontend(config)
    
    logger.info(f"Compiling {product_name}")
    stdout.info(f"Compiling {product_name}")
    os.makedirs("releases", exist_ok=True)

    OUTPUT_DIR = os.path.join(BUILD_FOLDER_NAME, version)

    if os.path.exists(OUTPUT_DIR):
        _continue = utils.prompt_select(title=f"Directory `releases/{version}` not empty, continue?", choices=[questionary.Choice(title="Yes", value=True),
            questionary.Choice(title="No", value=False)])       

        if not _continue:
            stdout.info("Operation cancelled by user.", exit=True)

    compile_executable(config, debug)

    if is_installable:
        exe_path = os.path.join(OUTPUT_DIR, f"{product_name}.exe")
        bundler.windows.msi.compile_installer(exe_path, config)

    end = time.perf_counter()
    timelapse = end - start
    stdout.info(f"Done in: {timelapse:.2f}s")

    if verbose:
        logger.setLevel(log.NOTSET)



def compile_executable(config, debug):
    version = config["version"]
    product_name = config["productName"]
    dist_dir = config["build"]["distDir"]

    build_dir = resolve_dist_dir(dist_dir)

    MODE = "--mode=" + "onefile"
    PYTHON_PATH = sys.executable if utils.is_venv() else "python"
    OUTPUT_DIR = "--output-dir=" + os.path.join(BUILD_FOLDER_NAME, version)
    INCLUDE_DIST = "--include-data-dir=" + f"{build_dir}={dist_dir}/"
    INCLUDE_CONFIG_FILE = "--include-data-files=" + f"{CONFIG_PATH}={CONF_FILENAME}"
    INCLUDE_LIB = "--include-data-files=" + f"{os.path.join(utils.get_package_path(LIB_NAME), "core", "dlls")}\\*={LIB_NAME}/core/dlls/"

    CONSOLE_MODE = "--windows-console-mode=" + ("force" if debug else "hide")

    ICON_PATH = Path(os.path.dirname(CONFIG_PATH)).resolve() / "icons" / "icon.ico"
    ICON = "--windows-icon-from-ico=" + str(ICON_PATH)
    INCLUDE_ICON = "--include-data-files=" + f"{ICON_PATH}=icon.ico"

    OUTPUT_NAME = "--output-filename=" + product_name
    PRODUCT_NAME = "--product-name=" + product_name
    COMPANY_NAME = "--company-name=" + product_name
    FILE_VERSION = "--file-version=" + version
    PRODUCT_VERSION = "--product-version=" + version

    cmd = [
        PYTHON_PATH, "-m", "nuitka", MODE,
        INCLUDE_DIST, INCLUDE_LIB, INCLUDE_CONFIG_FILE,
        OUTPUT_DIR, PRODUCT_NAME, COMPANY_NAME, FILE_VERSION,
        PRODUCT_VERSION, OUTPUT_NAME, CONSOLE_MODE,
        ICON, INCLUDE_ICON,
        ENTRY_POINT_PATH,
    ]

    try:
        subprocess.run(cmd, check=True)
    except subprocess.CalledProcessError as err:
        stdout.error(f"Failed to compile project: {err}", exit=True)


def resolve_dist_dir(name: str):
    current_dir = os.getcwd()
    if os.path.isdir(os.path.join(current_dir, name)):
        return os.path.join(current_dir, name)

    parent_dir = os.path.dirname(current_dir)
    if os.path.isdir(os.path.join(parent_dir, name)):
        return os.path.join(parent_dir, name)

    stdout.error("Failed to find build directory", exit=True)


def build_frontend(config: dict):
    build_command = config["build"]["beforeBuildCommand"]

    try:
        logger.info(f"Running {build_command} command")
        stdout.info(f"Running {build_command} command")
        subprocess.run(build_command, shell=True, check=True)
        stdout.info(f"Build complete")
    except subprocess.CalledProcessError as err:
        logger.error(f"Failed to build frontend: {err}")
        stdout.error(f"Failed to build: {err}", exit=True)



import json
import shutil
from pathlib import Path
from . import TEMPLATE_DIR
from jinja2 import Template
from nyvalis.tools import log
from typing import List, Dict, Any
from ...models import DependencyObj, FeaturePath

logger = log.get(log.LIB_NAME)

BASE_TAILWIND_DIR = Path(TEMPLATE_DIR) / "feats" / "tailwindcss"
BASE_ESLINT_DIR = Path(TEMPLATE_DIR) / "feats" / "eslint"
BASE_PRETTIER_DIR = Path(TEMPLATE_DIR) / "feats" / "prettier"

class FeaturesEnum:
    PRETTIER = DependencyObj(title="Prettier (Code formatter)", value="PRETTIER", 
                            packages={"prettier": "^3.4.2",
                                    "prettier-plugin-svelte": "^3.3.3",
                                    "prettier-plugin-tailwindcss": "^0.6.11",}, 
                            extras={})
    

    ESLINT = DependencyObj(title="ESLint (Code Linter)", value="ESLINT", 
                           packages={"@eslint/compat": "^1.2.5",
                            "@eslint/js": "^9.18.0", "eslint": "^9.18.0",
                            "eslint-config-prettier": "^10.0.1",
                            "eslint-plugin-svelte": "^2.46.1",},
                            extras={})
    

    TAILWINDCSS = DependencyObj(title="Tailwindcss", value="TAILWINDCSS", 
                                packages={"tailwindcss": "^4.0.0", "@tailwindcss/vite": "^4.0.0",}, 
                                extras={"prettier": "prettier-plugin-tailwindcss"})
    
def resolve(name: str, typecheck: DependencyObj, features: List[DependencyObj]):
    use_tailwind = any(feature.value == FeaturesEnum.TAILWINDCSS.value for feature in features)
    use_eslint = any(feature.value == FeaturesEnum.ESLINT.value for feature in features)
    use_prettier = any(feature.value == FeaturesEnum.PRETTIER.value for feature in features)

    extras = {}
    dependencies = {}
    filepaths: Dict[str, FeaturePath] = {}

    path = Path(name).resolve()

    if use_tailwind:
        dependencies.update(FeaturesEnum.TAILWINDCSS.packages)
        filepaths.update({
            "tailwind.config": FeaturePath(BASE_TAILWIND_DIR / "tailwind.config.js", Path(path) / "tailwind.config.js"),
            "+page.svelte": FeaturePath(BASE_TAILWIND_DIR / "+page.svelte", Path(path) / "src" / "routes" / "+page.svelte"),
            "vite.config.js": FeaturePath(BASE_TAILWIND_DIR / "vite.config.js", Path(path) / f"vite.config.{typecheck.value}"),
            "app.css": FeaturePath(BASE_TAILWIND_DIR / "app.css", Path(path) / "src" / "app.css"),
            ".prettierrc": FeaturePath(BASE_TAILWIND_DIR / ".prettierrc", Path(path) / ".prettierrc"),
        })

    if use_eslint:
        dependencies.update(FeaturesEnum.ESLINT.packages)
        filepaths.update({
            "eslint.config": FeaturePath(BASE_ESLINT_DIR / "eslint.config.js", Path(path) / "eslint.config.js"),
        })

    if use_prettier:
        dependencies.update(FeaturesEnum.PRETTIER.packages)
        filepaths.update({
            ".prettierrc": FeaturePath(BASE_PRETTIER_DIR / ".prettierrc", Path(path) / ".prettierrc"),
            ".prettierignore": FeaturePath(BASE_PRETTIER_DIR / ".prettierignore", Path(path) / ".prettierignore"),
        })
            
        if use_eslint:
            dependencies["eslint-config-prettier"] = "^10.0.1"


    logger.info("Copying feature files")
    for filepath in filepaths.values():
        filepath.dst.parent.mkdir(parents=True, exist_ok=True)
        shutil.copy(filepath.src, filepath.dst)

    if use_prettier and use_tailwind:
        prettier_file_path = filepaths.get(".prettierrc")
        resolve_prettier_file(prettier_file_path, use_tailwind)
    
    logger.info("Writing package.json")
    json_path = Path(path) / "package.json"
    with open(json_path, 'r') as file:
        config: Dict[str, Any] = json.load(file)

    config["name"] = name if name != "." else "My app"
    config["devDependencies"].update(dependencies)

    with open(json_path, 'w') as file:
        json.dump(config, file, indent=2)

    logger.info("package.json ready")

def resolve_prettier_file(path: FeaturePath, tailwind: bool):
	with open(path.dst, 'r', encoding='utf-8') as file:
		template_content = file.read()

	template = Template(template_content)
	rendered_content = template.render(tailwind=tailwind)

	with open(path.dst, 'w', encoding='utf-8') as file:
		file.write(rendered_content)



import json
from nyvalis.tools import log
from nyvalis.core.config import Config
from nyvalis.core.invoker import Invoker
from nyvalis.core import ROOT_PATH, LIB_NAME
from concurrent.futures import ThreadPoolExecutor

from System.Diagnostics import Process
from System import Uri, Convert, Action
from System.Windows.Forms import DockStyle
from Microsoft.Web.WebView2.WinForms import WebView2
from Microsoft.Web.WebView2.Core import (
    CoreWebView2WebResourceContext,
    CoreWebView2CustomSchemeRegistration,
    CoreWebView2EnvironmentOptions,
    CoreWebView2Environment
)
from System.IO import MemoryStream, SeekOrigin
from System.Collections.Generic import List



class Webview:
    logger = log.get(log.LIB_NAME)

    def __init__(self, form, config: Config, workers: int, state):
        self.config = config
        self.scheme = LIB_NAME
        self.webview = WebView2()
        self.thread_pool = ThreadPoolExecutor(max_workers=workers)
        self.logger.info(f"Running with {workers} workers")
        self.invoke_results = {}
        self.state = state
        
        
        self.form = form
        form.Controls.Add(self.webview)
        
        self.webview.Dock = DockStyle.Fill
        self.webview.BringToFront()

        custom_scheme_registration = CoreWebView2CustomSchemeRegistration(self.scheme)
        custom_scheme_registration.TreatAsSecure = True
        custom_scheme_registration.HasAuthorityComponent = True

        custom_scheme_registrations = List[CoreWebView2CustomSchemeRegistration]()
        custom_scheme_registrations.Add(custom_scheme_registration)

        options = CoreWebView2EnvironmentOptions(
                additionalBrowserArguments=None,
                language=None,
                targetCompatibleBrowserVersion=None,
                allowSingleSignOnUsingOSPrimaryAccount=False,
                customSchemeRegistrations=custom_scheme_registrations
        )

        environment = CoreWebView2Environment.CreateAsync(None, None, options).GetAwaiter().GetResult()

        self.webview.CoreWebView2InitializationCompleted += self.on_webview_initialized
        self.webview.EnsureCoreWebView2Async(environment)

        self.logger.info("Webview initialized")

    def on_webview_initialized(self, sender, args):
        """Configuration after webview is ready"""
        if not args.IsSuccess:
            self.logger.critical(f"Webview initialization failed: {str(args.InitializationException)}")
            return
        
        self.webview.CoreWebView2.AddWebResourceRequestedFilter(f"{self.scheme}://*", CoreWebView2WebResourceContext.All)
        self.webview.CoreWebView2.WebResourceRequested += self.handle_custom_scheme_request
        self.logger.info("WebResourceRequested handler registered")

        if "__compiled__" in globals():
            url = f"{self.scheme}://app/"
            self.webview.CoreWebView2.Navigate(url)
        else:
            self.load_url(self.config.build.dev_url)
        
        settings = sender.CoreWebView2.Settings
        settings.AreDevToolsEnabled = self.config.dev_tools
        settings.AreDefaultContextMenusEnabled = self.config.context_menu

        self.logger.info(f"Webview DevTools: {self.config.dev_tools}")
        self.logger.info(f"Webview ContextMenu: {self.config.context_menu}")

        sender.CoreWebView2.WebMessageReceived += self.on_invoke
        self.logger.info("WebMessageReceived event handler registered")

    def handle_custom_scheme_request(self, sender, event_args):
        self.logger.debug(f"Request received: {event_args.Request.Uri}")

        if not event_args.Request.Uri.startswith(f"{self.scheme}://"):
            return None

        try:
            uri = event_args.Request.Uri
            parsed_uri = Uri(uri)
            
            raw_path = parsed_uri.AbsolutePath
            path = raw_path[1:] if raw_path.startswith('/') else raw_path
            
            base_dir = ROOT_PATH
            full_path = (base_dir / self.config.build.dist_dir / path).resolve()

            if not str(full_path).startswith(str(base_dir)):
                self.logger.warning(f"Permission denied: {full_path}")
                raise Exception(f"Permission denied: {full_path}")

            static_extensions = {'.js', '.css', '.png', '.jpg', '.svg', '.json', '.woff', '.woff2', '.ico'}

            if full_path.is_dir():
                full_path = full_path / "index.html"
            elif not full_path.exists():
                if full_path.suffix in static_extensions:
                    raise FileNotFoundError(f"File not found: {full_path}")
                full_path = base_dir / "index.html"
            if not full_path.is_file():
                raise FileNotFoundError(f"Not a file: {full_path}")

            mime_types = {
                '.html': 'text/html',
                '.js': 'application/javascript',
                '.mjs': 'application/javascript',
                '.css': 'text/css',
                '.png': 'image/png',
                '.jpg': 'image/jpeg',
                '.svg': 'image/svg+xml',
                '.json': 'application/json',
                '.woff': 'font/woff',
                '.woff2': 'font/woff2',
                '.ico': 'image/x-icon'
            }

            ext = full_path.suffix.lower()
            content_type = mime_types.get(ext, 'application/octet-stream')

            self.logger.info(f"Serving: {full_path}: {content_type}")

            with open(full_path, "rb") as file:
                content = file.read()

            stream = MemoryStream(content)
            stream.Seek(0, SeekOrigin.Begin)
            
            response = self.webview.CoreWebView2.Environment.CreateWebResourceResponse(
                stream,
                200,
                "OK",
                f"Content-Type: {content_type}; charset=utf-8" if 'text/' in content_type else f"Content-Type: {content_type}"
            )
            event_args.Response = response

        except FileNotFoundError as err:
            self.logger.warning(f"File not found: {str(err)}")
            error_msg = "404 Not Found".encode("utf-8")
            error_stream = MemoryStream(error_msg)
            error_stream.Seek(0, SeekOrigin.Begin)
            
            response = self.webview.CoreWebView2.Environment.CreateWebResourceResponse(
                error_stream,
                404,
                "Not Found",
                "Content-Type: text/plain; charset=utf-8"
            )
            event_args.Response = response
            
        except Exception as err:
            self.logger.error(f"Error: {str(err)}", exc_info=True)
            error_msg = f"500 Server Error: {str(err)}".encode("utf-8")
            error_stream = MemoryStream(error_msg)
            error_stream.Seek(0, SeekOrigin.Begin)
            
            response = self.webview.CoreWebView2.Environment.CreateWebResourceResponse(
                error_stream,
                500,
                "Internal Server Error",
                "Content-Type: text/plain; charset=utf-8"
            )
            event_args.Response = response

    def on_invoke(self, sender, args):
        """Called when client requests a invoke"""
        try:
            message = args.WebMessageAsJson

            if message is None:
                self.logger.error("Invalid message")
                return
            
            payload = json.loads(message)

            request_name = payload.get("command")
            request_params = payload.get("params")
            request_id = payload.get("id")

            command = Invoker.commands.get(request_name)

            if not command or not command.allowed:
                self.logger.error(f"Command `{request_name}` not found.")
                return
            
            self.logger.info(f"Command called: `{request_name}`")
            
            self.thread_pool.submit(
                self.process_command,
                command, request_params, request_id
            )

        except Exception as err:
            self.logger.error(f"Failed to process invoke: {err}")

    def process_command(self, command, params, request_id):
        """Process the invoke in a different thread"""
        try:
            response = Invoker.process(command=command, params=params, state=self.state)
            self.invoke_results[request_id] = response
            

            self.webview.Invoke(
                Action(lambda: self.send_response(request_id))
            )
            
        except Exception as err:
            self.logger.error(f"Failed to run invoke: {err}")

    def send_response(self, request_id):
        """Send the response of invoke_results to the client"""
        try:
            if request_id in self.invoke_results:
                result = self.invoke_results.pop(request_id)

                data = json.loads(result)

                response_data = {
                    "id": request_id,
                    "data": data["data"],
                    "error": data["error"]
                }

                self.webview.CoreWebView2.PostWebMessageAsString(json.dumps(response_data))
                self.logger.info(f"Response sent to {request_id}")
        except Exception as err:
            self.logger.error(f"Failed to sen response: {err}")

    def load_url(self, url):
        """Loads the url into the window"""
        self.webview.Source = Uri(url)
        self.logger.info("Client URL loaded into webview")

    def load_html(self, content):
        with open(r"C:\Vault\Code\pc\testes\build\index.html", 'r', encoding='utf-8') as file:
            content = file.read()
        
        self.webview.CoreWebView2.NavigateToString(content)

    def on_exit(self):
        """Terminates the webview process"""

        self.thread_pool.shutdown(wait=True)
        process_id = Convert.ToInt32(self.webview.CoreWebView2.BrowserProcessId)
        process = Process.GetProcessById(process_id)
        self.webview.Dispose()
        process.WaitForExit(3000)

        self.logger.info("Webview process terminated")

    def __del__(self):
        self.thread_pool.shutdown(wait=True)



import queue
import logging
from nyvalis.core import LIB_NAME
from typing import Dict, Optional
from nyvalis.tools.log import handlers

DEBUG = logging.DEBUG
INFO = logging.INFO
WARNING = logging.WARNING
ERROR = logging.ERROR
CRITICAL = logging.CRITICAL
NOTSET = logging.NOTSET
DISABLED = 100000

DEFAULT_DATEFMT = "%d/%m/%Y %H:%M:%S"
DEFAULT_FMT = "%(asctime)s %(name)s %(levelname)s - %(message)s"


class Env:
    name: str = "app"
    _fmt: str | None = None
    _datefmt: str | None = None
    _iso: bool = False
    _colored: bool = False

    @classmethod
    def init(cls, name: str = "app"):
        cls.name = name
        logger = logging.getLogger(name)
        logger.setLevel(logging.DEBUG)
        return cls
    
    @classmethod
    def __get_handler(cls, handler_type):
        logger = logging.getLogger(cls.name)
        for handler in logger.handlers:
            if isinstance(handler, handler_type):
                return handler
        return None

    @classmethod
    def level(cls, value: int):
        logging.getLogger(cls.name).setLevel(value)
        return cls
    
    @classmethod
    def fmt(cls, value: str):
        cls._fmt = value
        return cls
    
    @classmethod
    def datefmt(cls, value: int):
        cls._datefmt = value
        return cls
    
    @classmethod
    def colored(cls, value: bool = True):
        cls._colored = value
        return cls
    
    @classmethod
    def iso(cls, value: bool = True):
        cls._iso = value
        return cls
    
    @classmethod
    def file(cls, path: str, fmt: str | None = None, datefmt: str | None = None, 
             json: bool = False, json_fmt: Optional[Dict[str, str]] = None, encoding: str = "utf-8", iso: bool = False):
        
        logger = logging.getLogger(cls.name)
        file_handler = Env.__get_handler(logging.FileHandler)

        iso = iso or cls._iso
        fmt = fmt or cls._fmt or DEFAULT_FMT
        datefmt = datefmt or cls._datefmt or DEFAULT_DATEFMT

        formatter = handlers.CustomFormatter(fmt, datefmt, colored=False, iso=iso)

        if json:
            formatter = handlers.JSONFormatter(json_fmt=json_fmt, iso=iso)

        if file_handler:
            file_handler.setFormatter(formatter)
        else:
            stream_handler = logging.FileHandler(path, encoding=encoding)
            stream_handler.setFormatter(formatter)
            logger.addHandler(stream_handler)
        
        return cls

    @classmethod
    def stdout(cls, fmt: str | None = None, datefmt: str | None = None, iso: bool = False, colored: bool = True):
        logger = logging.getLogger(cls.name)
        stream_handler = Env.__get_handler(logging.StreamHandler)

        iso = iso or cls._iso
        colored = colored or cls._colored
        fmt = fmt or cls._fmt or DEFAULT_FMT
        datefmt = datefmt or cls._datefmt or DEFAULT_DATEFMT

        formatter = handlers.CustomFormatter(fmt, datefmt, colored=colored, iso=iso)

        if stream_handler:
            stream_handler.setFormatter(formatter)
        else:
            stream_handler = logging.StreamHandler()
            stream_handler.setFormatter(formatter)
            logger.addHandler(stream_handler)
        return cls
    
    @classmethod
    def queue(cls):
        from logging.handlers import QueueHandler, QueueListener
        logger = logging.getLogger(cls.name)
        
        if Env.__get_handler(QueueHandler):
            return cls
        
        cls.log_queue = queue.Queue()
        cls.queue_handler = QueueHandler(cls.log_queue)
        
        stream_handler = cls.__get_handler(logging.StreamHandler)
        file_handler = cls.__get_handler(logging.FileHandler)

        if stream_handler or file_handler:
            for handler in logger.handlers[:]:
                logger.removeHandler(handler)
        
        logger.addHandler(cls.queue_handler)

        cls.queue_listener = QueueListener(cls.log_queue, *[handler for handler in [stream_handler, file_handler] if handler])
        cls.queue_listener.start()

        return cls
    
    def __del__(self):
        if hasattr(self, 'queue_listener') and self.queue_listener:
            self.queue_listener.stop()
    
def log(level: int, msg: str, *args, stacklevel=2, **kwargs):
    logging.getLogger(Env.name).log(level, msg, *args, stacklevel=kwargs.pop("stacklevel", stacklevel), **kwargs)
    
def info(msg: str, *args, stacklevel=2, **kwargs):
    logging.getLogger(Env.name).info(msg, *args, stacklevel=kwargs.pop("stacklevel", stacklevel), **kwargs)

def debug(msg: str, *args, stacklevel=2, **kwargs):
    logging.getLogger(Env.name).debug(msg, *args, stacklevel=kwargs.pop("stacklevel", stacklevel), **kwargs)

def warning(msg: str, *args, stacklevel=2, **kwargs):
    logging.getLogger(Env.name).warning(msg, *args, stacklevel=kwargs.pop("stacklevel", stacklevel), **kwargs)

def error(msg: str, *args, stacklevel=2, **kwargs):
    logging.getLogger(Env.name).error(msg, *args, stacklevel=kwargs.pop("stacklevel", stacklevel), **kwargs)

def exception(msg: str, *args, stacklevel=2, **kwargs):
    logging.getLogger(Env.name).exception(msg, *args, stacklevel=kwargs.pop("stacklevel", stacklevel), **kwargs)

def critical(msg: str, *args, stacklevel=2, **kwargs):
    logging.getLogger(Env.name).critical(msg, *args, stacklevel=kwargs.pop("stacklevel", stacklevel), **kwargs)

def get(name: str):
    return logging.getLogger(name)
    

def set_lib_level(level: int):
    logging.getLogger(LIB_NAME).setLevel(level)


import os
import click
import questionary
from nyvalis.tools import stdout, log
from nyvalis.cli.commands import utils
from nyvalis.cli.commands.models import PackageManager, PackageManagerEnum, TemplateObj, BuildField

FULL_PATH = None
logger = log.get(log.LIB_NAME)

class TemplateEnum:
    from nyvalis.cli.commands.new.svelte import create_svelte_app
    from nyvalis.cli.commands.new.react import create_react_app
    
    SVELTE = TemplateObj(title="Svelte", value="SVELTE", create=create_svelte_app, 
                        build_field=BuildField("http://localhost:5173", "build", r"{{ manager }} dev", r"{{ manager }} build"))
    
    REACT = TemplateObj(title="React", value="REACT", create=create_react_app, 
                        build_field=BuildField("http://localhost:5173", "build", r"{{ manager }} dev", r"{{ manager }} build"))

@click.command()
@click.option('--verbose', '-v', is_flag=True, help="Change lib logger level to DEBUG")
@click.option('--skip-venv', is_flag=True, help="Skip installing venv")
@click.option('--skip-pip', is_flag=True, help="Skip installing pip")
@click.option('--skip-node', is_flag=True, help="Skip node install")
@click.argument("name", required=False)
def new(verbose, skip_venv, skip_pip, skip_node, name: str = None):
    """Creates a new project

    name: The name of the project. If not provided, you will be asked to enter one.
    """

    global FULL_PATH

    if verbose:
        logger.setLevel(log.DEBUG)
    
    if not name:
        name = utils.prompt_text("Project name", default=".")
        logger.info(f"Project name: {name}")

    FULL_PATH = os.path.join(os.getcwd(), name)

    if os.path.exists(FULL_PATH):
        if os.listdir(FULL_PATH):

            stop = utils.prompt_select(title="Directory not empty, continue?", choices=[questionary.Choice(title="Yes", value=False),
                questionary.Choice(title="No", value=True)])
            
            if stop:
                logger.info(f"Operation cancelled by user.")
                stdout.error("Operation cancelled by user.", exit=True)

    manager: PackageManager = utils.prompt_select(title="Choose the package manager", choices=[questionary.Choice(title="npm", value=PackageManagerEnum.NPM),
        questionary.Choice(title="pnpm", value=PackageManagerEnum.PNPM)])

    scaffold_template(manager, name)

    venv = False
    if not skip_venv:
        if not os.path.exists(os.path.join(FULL_PATH, ".venv")):
            venv: bool = utils.prompt_select(title="Would you like to set up venv?", choices=[questionary.Choice(title="Yes", value=True),
                questionary.Choice(title="No", value=False)])
            
            if venv:
                utils.setup_venv(name)

    
    if not skip_pip:
        pip: bool = utils.prompt_select(title="Would you like to run pip install?", choices=[questionary.Choice(title="Yes", value=True),
            questionary.Choice(title="No", value=False)])

        if pip:
            utils.run_pip_install(name, venv)

    if not skip_node:
        if not os.path.exists(os.path.join(name, "node_modules")):
            node_modules = utils.prompt_select(title=f"Would you like to run {manager.cli} install?", choices=[questionary.Choice(title="Yes", value=True),
                questionary.Choice(title="No", value=False)])
        
            if node_modules:
                utils.run_npm_install(name, manager)

    stdout.info("Project setup completed successfully!")


def scaffold_template(manager: PackageManager, path: str):
    template: TemplateObj = utils.prompt_select(title="Choose your desired UI framework", choices = [
                                questionary.Choice(title=tc.title, value=tc) 
                                for tc in TemplateEnum.__dict__.values() 
                                if isinstance(tc, TemplateObj)
                            ]) 

    if not utils.is_cli_installed("npm"):
        stdout.error("NPM is not installed. Install at https://nodejs.org/en", exit=True)

    template.create(path)

    if not os.path.exists(os.path.join(FULL_PATH, ".git")):
        create_repo = utils.prompt_select(
            title=f"Create git repository?",
            choices=[
                questionary.Choice(title="Yes", value=True),
                questionary.Choice(title="No", value=False)
            ]
        )

        if create_repo:
            if utils.is_cli_installed("npm"):
                utils.create_git_repo(FULL_PATH)
            else:
                stdout.error("Git is not installed. Install at https://git-scm.com/", exit=True)

    utils.write_requirements(path)
    utils.scaffold_python(manager, path, template)



import os
import sys
import click
import subprocess
from nyvalis.tools import stdout, log
from nyvalis.core import CONF_FILENAME

logger = log.get(log.LIB_NAME)

@click.command()
@click.option('--verbose', '-v', is_flag=True, help="Change lib logger level to DEBUG")
def run(verbose):
    """Runs the project for development
    """

    run_application(verbose)

    os.chdir("src-python")

    run_application(verbose)


def run_application(verbose: bool):
    if os.path.isfile("main.py"):
        if os.path.isfile(CONF_FILENAME):
            try:
                if verbose:
                    subprocess.run([sys.executable, "main.py", "-v"], check=True)
                else:
                    subprocess.run([sys.executable, "main.py"], check=True)
            except subprocess.CalledProcessError:
                stdout.error("Failed to run application")
                return
        else:
            stdout.error(f"Could not find application.")


import os
import shutil
import questionary
from typing import List
from pathlib import Path
from nyvalis.tools import log
from ...models import DependencyObj

logger = log.get(log.LIB_NAME)
TEMPLATE_DIR = os.path.join(Path(__file__).resolve().parents[3], "templates", "options", "svelte")

def create_svelte_app(path: str):
    from . import features, typecheck
    from nyvalis.cli.commands.utils import prompt_select, prompt_checkbox
    
    _typecheck: DependencyObj = prompt_select(title="Choose your type check",
        choices=[questionary.Choice(title=option.title, value=option) for option in typecheck.TypecheckEnum.__dict__.values() if isinstance(option, DependencyObj)]) 
    
    _features: List[DependencyObj] = prompt_checkbox(title="Choose your desired features", choices=[
        questionary.Choice(title=option.title, value=option)
        for option in features.FeaturesEnum.__dict__.values()
        if isinstance(option, DependencyObj)
        ])
    
    logger.info(f"typecheck: {_typecheck}")
    logger.info(f"features: {_features}")
    
    src_path = Path(TEMPLATE_DIR) / "base"
    dst_path = Path(os.getcwd()) / path

    logger.info(f"Scaffolding tempalte")
    shutil.copytree(src_path, dst_path, dirs_exist_ok=True)
    logger.info(f"Template ready")

    typecheck.resolve(path, _typecheck, _features)
    features.resolve(path, _typecheck, _features)


from pathlib import Path
from dataclasses import dataclass
from typing import Callable, Dict, Any


@dataclass
class BuildField:
    dev_url: str
    dist_dir: str
    dev_command: str
    build_command: str

    def to_dict(self):
        return {
            "devUrl": self.dev_url,
            "distDir": self.dist_dir,
            "beforeBuildCommand": self.build_command,
            "beforeDevCommand": self.dev_command,
        }


@dataclass
class TemplateObj:
    title: str
    value: str
    create: Callable
    build_field: BuildField


@dataclass
class FeaturePath:
    src: Path
    dst: Path


@dataclass
class DependencyObj:
    title: str
    value: str
    packages: Dict[str, str]
    extras: Dict[str, Any]
 

@dataclass
class PackageManager:
    execute: str
    cli: str

class PackageManagerEnum:
    NPM = PackageManager(execute="npm run", cli="npm")
    PNPM = PackageManager(execute="pnpm", cli="npm")
    